#include <iolib.h>
#include <memlib.h>
#include <mmc.h>
#include <msdos.h>
#include <msgqlib.h>
#include <multex.h>
#include <string.h>
#include <crt.h>
#include <i2c.h>
#include <uart.h>
#include <spi.h>
#include <timer-arm.h>
#include <pwm.h>
#include <pll.h>
#include <led.h>
#include <irq.h>
#include <ve.h>
#include <io.h>

#ifdef BOARD_IS_BEAGLE
#include <board/beagle.h>
#endif
#ifdef BOARD_IS_BONE
#include <board/beaglebone.h>
#endif
#ifdef BONE_IS_IGEP
#include <board/igep0033.h>
#endif

#include <cache.h>
#include <console.h>
#include <drv/netcard/lan9514drv.h>
#include <drv/netcard/sunxi_gmac.h>
#include <netdevice.h>
#include <netinet.h>
#include <shell.h>
// #include <siodrv.h>
#include <stdio.h>
#include <timer.h>
#include <gpio.h>

#ifdef CONFIG_PATH
#include CONFIG_PATH
#else
#include "config.h"
#endif


#include <intlib.h>
// #include <multimedia/sunxi-codec.h>
#include <sound.h>
#include <uartconfig.h>

// Конфигурация аппаратной части при старте ядра.
#include <arch/arch.h>
#include <arch/archdef.h>

extern void sesam_init (void);
extern void sesam_init_sdc3 (void);


void mountMmcDisks () {

#ifdef INCLUDE_DOSFS

    dosInit ();
    mmcMount (0, 0);

    bool archOk;
    const char *cpu = archGetString (ARCH_CPU, &archOk);

    if (archOk & archCheckString (cpu, ARCH_PROC_A20)) {
        // Настройка на модуль SE-SOM, подключение карты памяти-2 (mmcMount (2))
        sesam_init ();
#   ifdef SESAM_INIT_SDC3
        sesam_init_sdc3 ();
#   endif
    }




#ifdef INCLUDE_SATA
    createSATA ();
#endif

#endif  //  INCLUDE_DOSFS
}


// Конфигурация модулей до старта ядра.
static void archPreInit (void) {

    archCreate ();

    // ДОБАВЛЕНИЕ ТИПА ПРОЦЕССОРА!
    archAddString (ARCH_CPU, ARCH_PROC);

    // Модули, которым нужно знать только тип процессора.
    irqArchInit ();
    pllArchInit ();
    gpioArchInit ();
    taArchInit ();
    soundArchInit ();
    mmcArchInit ();
    veArchInit ();
    i2cArchInit ();
    spiArchInit ();
    uartArchInit ();

}

// Добавление особенностей известных плат,
// конфигурация модулей после старта ядра.
static void archPostInit () {
    bool ok;
    
    // Если известен тип платы - можно дополнить известные параметры
    const char *board = archGetString (ARCH_BOARD, &ok);

    // Особенности плат - процессор, разводка

    if (ok && archCheckString (board, ARCH_BOARD_SE8350_00)) {
        // archAddString (ARCH_CPU, ARCH_PROC_V3S);
    } else
    if (ok && archCheckString (board, ARCH_BOARD_SE8351_00)) {
        // archAddString (ARCH_CPU, ARCH_PROC_V3S);
        archAddInt (ARCH_BACKLIGHT_FREQUENCY, 100, "Hz");
    } else
    if (ok && archCheckString (board, ARCH_BOARD_SE_DB_A20_B254)) {
        // archAddString (ARCH_CPU, ARCH_PROC_A20);
    }

    // Отладка:
    // Сбросить текущую конфигурацию в файл - отладка
    // mapStore ("current.arc", arch);

#ifdef DEBUG
    // Печать конфигурации в консоль.
    archPrint ();
#endif

// НАСТРОЙКА МОДУЛЕЙ после составления полного списка конфигурации
    pwmArchInit (); /// @todo Добавить A40 (переписать драйвер под А40)
    ledArchInit ();


}


extern void USER_PROC (int);

int usb_init ();

#ifdef DRAW_LOGO
extern void DRAW_LOGO ();
#endif
#ifdef USER_PROC
extern void USER_PROC ();
#endif

extern int sunxiLanDrvInit (struct netDevice *dev);
extern void tcpConsole ();

extern void cubie_init (void);
extern void cbt_init (void);
extern void banana_init (void);
extern void orange_init (void);
extern void bpi_m2u_init (void);
extern void cache_enable_all (void);
extern void dcache_disable (void);
extern void lichee_init (void);
extern void sea40_init(void);
/// @todo Убрать переменную в a40graph
int cs_is_rgb = 1; // для a40graph!

extern STATUS ncDevCreate (char *name);
extern int netLoaderInit (void);
extern STATUS initSockLib (void);
extern int FTP (void);
extern void ftp_AllowToChangeWorkDrive (void);
extern void ftp_ShowSystemFiles (void);
extern void initFP (void);

int consoleFd;

//! Flag, which disables output to stdout. True by default. Should be enabled after correct console is prepared.
extern volatile bool DisableEarlySerialConsoleOutput;



#ifndef KEYBOARD_DONT_USE_STDIN

extern MSG_Q_ID getConsoleInQ ();

MSG_Q_ID getInQueueForKeyboard () {
    return getConsoleInQ ();
}

#else

static MSG_Q_ID UsbKeyboardQueue = NULL;

MSG_Q_ID getInQueueForKeyboard () {
    return UsbKeyboardQueue;
}

#endif  // KEYBOARD_DONT_USE_STDIN

#ifdef REDIRECT_STDOUT_TO_FILE

#ifndef STDOUT_FILE
#define STDOUT_FILE "C:/STDOUT.TXT"
#endif

static void flushStdOutFileTask (int fd) {
    for (;;) {
        flush (fd);
        taskDelay (sysClkRateGet () * 10);
    }
}

#endif  // REDIRECT_STDOUT_TO_FILE

// __attribute__ ((unused)) static int shellTaskFunc (int printHdr) {
//     shell (printHdr);
//     return OK;
// }

//-------------------------------------------------
//                  usr_init


int usr_init (__attribute__ ((unused)) int d) {

#if defined(INCLUDE_SIO_CONSOLE) || (defined(INCLUDE_NETINET) && defined(INCLUDE_NET_CONSOLE))
    int fd;
#endif

#ifdef INCLUDE_NETINET
    NetInputFunc = NULL;
#endif

#ifdef INCLUDE_SIO_CONSOLE
#if defined(RECONFIGURE_CONSOLE_UART_COM_NUM) & defined(RECONFIGURE_CONSOLE_UART_SETUP_TYPE)
    uart_ConfigureHardware (RECONFIGURE_CONSOLE_UART_COM_NUM, RECONFIGURE_CONSOLE_UART_SETUP_TYPE);
    CONSOLE_BASE = uart_GetBase (RECONFIGURE_CONSOLE_UART_COM_NUM);
    CONSOLE_IRQ = uart_GetIrq (RECONFIGURE_CONSOLE_UART_COM_NUM);
#endif
    // sioConCreate (CONSOLE_BASE, CONSOLE_IRQ, 115200);
    uartInit (UART_0, UART_GPIO_DEFAULT);
    uartSetBaudRate (UART_0, UART_BAUD_115200);
    uartSetTextMode (UART_0, true);

    // fd = open ("sioCon", 0);
    fd = open (uartName (UART_0), O_RDWR);
    ioGlobalStdSet (0, fd);
    ioGlobalStdSet (1, fd);
    ioGlobalStdSet (2, fd);
    consoleFd = fd;
    // Приостановка задачи, чтобы консоль подцепилась
    // (иначе не идут printf).
    taskDelay (1);


    /// @todo delete test
    gpio_direction_output (P_C + 0, 1);


    // Если предыдущая загрузка не удачная, то в консоли
    // может быть какой-угодно цвет.
    textColor (clNone);
    printf ("Serial console started\n");
#endif
#ifdef DISABLE_EARLY_SERIAL_CONSOLE_OUTPUT
    DisableEarlySerialConsoleOutput = false;
#endif

#ifdef KEYBOARD_DONT_USE_STDIN
    UsbKeyboardQueue = msgQCreate (256, sizeof (char), MSG_Q_FIFO);
#endif  // KEYBOARD_DONT_USE_STDIN

    
// #ifdef INCLUDE_DOSFS
//     dos = dosInit ();
//     dosDrv = dos;
//     createMMC (0);
// #ifdef BOARD_IS_BONE

// #ifndef BONE_IS_IGEP
//     createMMC (1);
// #endif
// #ifdef INCLUDE_SATA
//     createSATA ();
// #endif
// #endif

// #endif  // DOSFS

#ifdef INIT_GPMC
    printf ("Init GPMC bus...");
    gpmc_init ();
    printf ("done.\n");
#endif

    // Монтирование всех дисков - до чтения архитектуры из файлов
    mountMmcDisks ();

// ARCH
// Инит аппаратной части - определение архитектуры. (обращается ко всем дискам).
    archMainInit ();
    archPostInit ();
    // Определение процессора:
    bool archOk;
    const char *cpu = archGetString (ARCH_CPU, &archOk);


#ifdef BOARD_IS_BEAGLE
    beagle_init ();
#endif
#ifdef BOARD_IS_OVERO
    overo_init ();
#endif
#ifdef BOARD_IS_IGEP0030
    igep0030_init ();
#endif
#ifdef BOARD_IS_BONE
    bbb_init ();
#ifdef BONE_IS_IGEP
    igep0033_init ();
#endif
#endif
#ifdef BOARD_IS_CUBIE
    cubie_init ();
#endif
#ifdef BOARD_IS_CUBIETRUCK
    cbt_init ();
#endif
#ifdef BOARD_IS_A13
    a13_init ();
#ifdef ARTEFACT
    arte_init ();
#endif
#endif

/// @todo Переводить остальные платы на arch
if (archOk & archCheckString (cpu, ARCH_PROC_H3)) {
    orange_init ();
}
// #ifdef BOARD_IS_ORANGE
//     orange_init ();
// #endif

// Здесь в монтирование диска 2 для SE-SOM - переехало в mountMmcDisks
// if (archOk & archCheckString (cpu, ARCH_PROC_A20)) {
//     sesam_init ();
// #   ifdef SESAM_INIT_SDC3
//     sesam_init_sdc3 ();
// #   endif
// }


if (archOk & archCheckString (cpu, ARCH_PROC_A40)) {
    sea40_init ();
}


// #ifdef BOARD_IS_SESAM
//     sesam_init ();
// #ifdef SESAM_INIT_SDC3
//     sesam_init_sdc3 ();
// #endif
// #endif

#ifdef BOARD_IS_BANANA
    banana_init ();
#endif
#ifdef BOARD_IS_BPI_M2U
    bpi_m2u_init ();
#endif
#ifdef BOARD_IS_V5
    lindeni_init ();
#endif

/// @todo Переводить остальные платы на arch
if (archOk & archCheckString (cpu, ARCH_PROC_V3S)) {
    lichee_init ();
}
// #ifdef BOARD_IS_LICHEE
//     lichee_init ();
// #endif

#ifdef DCACHE_ENABLE
    cache_enable_all ();
#endif

#ifdef CLEAN_MEMORY
    printf ("Cleanup memory pool...");
    memClean ();
    printf ("done.\n");
#endif

#ifdef INCLUDE_USB
#ifdef INIT_USB_BEFORE_LOGO
    usb_init ();
#endif
#endif

#ifdef DRAW_LOGO
    DRAW_LOGO ();
#endif

// #ifdef INCLUDE_I2C
//     // i2c_init ();
//     i2cInit (I2C_0, I2C_GPIO_DEFAULT, 400000);
// #endif

#ifdef INCLUDE_USB
#ifndef INIT_USB_BEFORE_LOGO
    usb_init ();
#endif
#endif

#ifdef INCLUDE_NETINET
    bool freeIpToSetVar = false;
    char *ipToSet = IP_ADDRESS;
#if defined(CHECK_PRIMARY_IP_ADDRESS)
    ip_addr_t primaryIp = getPrimaryIpAddress ();
    if (primaryIp != 0) {
        ipToSet = malloc (3 * 4 + 3 + 1);
        iptostr (primaryIp, ipToSet);
        freeIpToSetVar = true;
    }
#endif /* defined(CHECK_PRIMARY_IP_ADDRESS) */

#ifdef BOARD_IS_BEAGLE
    netInit (ipToSet, lan9514DrvInit);
#endif
#ifdef BOARD_IS_OVERO
    netInit (ipToSet, smc911xDrvInit);
#endif
#ifdef BOARD_IS_IGEP0030
    netInit (ipToSet, smc911xDrvInit);
#endif
#ifdef BOARD_IS_BONE
    netInit (ipToSet, CPSWDrvInit);
#endif
#ifdef BOARD_IS_CUBIE
#ifdef ETH_IS_GMAC
    netInit (ipToSet, sunxiGmacDrvInit);
#else
    netInit (ipToSet, sunxiLanDrvInit);
#endif
#endif

/// @todo Переводить остальные платы на arch
if (archOk & archCheckString (cpu, ARCH_PROC_A20)) {
#   ifdef ETH_IS_GMAC
    netInit (ipToSet, sunxiGmacDrvInit);
#   else
    netInit (ipToSet, sunxiLanDrvInit);
#   endif
}
// #ifdef BOARD_IS_SESAM
// #ifdef ETH_IS_GMAC
//     netInit (ipToSet, sunxiGmacDrvInit);
// #else
//     netInit (ipToSet, sunxiLanDrvInit);
// #endif
// #endif

#ifdef BOARD_IS_CUBIETRUCK
    netInit (ipToSet, sunxiGmacDrvInit);
#endif
#ifdef BOARD_IS_A13
    netInit (ipToSet, lan9514DrvInit);
#endif

/// @todo Переводить остальные платы на arch
if (archOk & archCheckString (cpu, ARCH_PROC_H3)) {
    netInit (ipToSet, sunxiGethDrvInitH3);
}
// #ifdef BOARD_IS_ORANGE
//     netInit (ipToSet, sunxiGethDrvInitH3);
// #endif

#ifdef BOARD_IS_BANANA
    netInit (ipToSet, sunxiGmacDrvInit);
#endif
#ifdef BOARD_IS_BPI_M2U
    netInit (ipToSet, sunxiGethDrvInitA40);
#endif

/// @todo Переводить остальные платы на arch
if (archOk & archCheckString (cpu, ARCH_PROC_V3S)) {
    netInit (ipToSet, sunxiGethDrvInitH3);
}
// #ifdef BOARD_IS_LICHEE
//     netInit (ipToSet, sunxiGethDrvInitH3);
// #endif


    if (freeIpToSetVar) {
        free (ipToSet);
    }

#ifdef INCLUDE_NET_CONSOLE
    {
        ncDevCreate ("netCon");
        fd = creat ("netCon", 0);
#if !defined(INCLUDE_SIO_CONSOLE)
        ioGlobalStdSet (0, fd);  // stdin
        ioGlobalStdSet (1, fd);  // stdout
        ioGlobalStdSet (2, fd);  // stderr
#else
        TASK_ID netShell = taskSpawn ("netShell", 255, 0, 0,
                                      (FUNCPTR)shellTask, 1);
        ioTaskStdSet (netShell, 0, fd);
        ioTaskStdSet (netShell, 1, fd);
        ioTaskStdSet (netShell, 2, fd);
        taskPrioritySet (netShell, 5);
#endif
    }

#endif  // NET_CONSOLE
#ifdef INCLUDE_NETLOADER
    netLoaderInit ();
#endif
#ifdef INCLUDE_TCP
    initSockLib ();
#ifdef FTP_SERVER
    FTP ();
#endif
#ifdef INCLUDE_TCP_CONSOLE
    tcpConsole ();
#endif
#endif
#endif  // NETINET

#ifdef FTP_ALLOW_TO_CHANGE_WORK_DRIVE
    ftp_AllowToChangeWorkDrive ();
#endif
#ifdef FTP_SHOW_ALL_FILES
    ftp_ShowSystemFiles ();
#endif

#ifdef REDIRECT_STDOUT_TO_FILE

    if (fileExists (STDOUT_FILE))
        remove (STDOUT_FILE);
    int stdOutFile = open (STDOUT_FILE, O_WRONLY | O_CREAT);

    ioGlobalStdSet (0, stdOutFile);  // stdin
    ioGlobalStdSet (1, stdOutFile);  // stdout
    ioGlobalStdSet (2, stdOutFile);  // stderr
    taskSpawn ("FileConFlusher", 10, 0, 0, (FUNCPTR)flushStdOutFileTask, stdOutFile);

#endif

// #ifdef INCLUDE_SOUND
// #ifdef SYSTEM_IS_SUNXI
//     initSoundSystem (initSunxiAudio);
// #endif
// #endif

#ifdef USER_PROC
    USER_PROC (1);
#else
    shell (1);
#endif
    taskDelay (WAIT_FOREVER);
    return 0;
}

extern int rstctl;
int dummy_nand;

#ifndef MEM_POOL_START
#define MEM_POOL_START 0
#endif

void start_ccode (void) {

#ifdef NANDBOOT
    volatile int *UARTSR = (int *)(0x1c2807c);
    volatile int *MMC0PINCFG = (int *)(0x1c208B4);
#endif
    disable_interrupts ("start_ccode");
#ifdef NANDBOOT
    dummy_nand = *UARTSR;
    //*MMC0PINCFG = 0x00222222;
#endif
    rstctl = 0;// PRM_RSTCTRL;
    //  led_off(0);
#ifndef DISABLE_EARLY_SERIAL_CONSOLE_OUTPUT
    DisableEarlySerialConsoleOutput = false;
#endif

    puts ("Multex Kernel started\n");
    dcache_disable ();
    // led_on(0);
#ifdef INIT_PRCM
    init_prcm ();
#endif

#ifdef INCLUDE_NEON
    initFP ();
#endif
    // led_on(0);
    // puts("kernelInit MEM_POOL_START ");putl(MEM_POOL_START);
    // puts("  MEM_POOL_END ");putl(MEM_POOL_END);
    // puts("\n");

// Старт ядра
    
    kernelInit (&archPreInit, usr_init, 65536, MEM_POOL_START, MEM_POOL_END);
}
