#ifdef CONFIG_PATH
#include CONFIG_PATH
#else
#include "config.h"	
#endif
.text
.code 32 
.globl _start
_start: 
    b reset
	ldr	pc, _undefined_instruction
	ldr	pc, _software_interrupt
	ldr	pc, _prefetch_abort
	ldr	pc, _data_abort
	ldr	pc, _not_used
	ldr	pc, _irq
	ldr	pc, _fiq

_reset: 			   	   .word reset
_undefined_instruction:    .word undefined_instruction
_software_interrupt:	   .word software_interrupt
_prefetch_abort:		   .word prefetch_abort
_data_abort:			   .word data_abort
_not_used:				   .word not_used
_irq:					   .word irq_handle
_fiq:					   .word fiq
_pad:					   .word 0x12345678 /* now 16*4=64 */

.global _end_vect
.global _irq_stack
.global _bss_end
_end_vect:

.balignl 16,0xdeadbeef

/*  Вспомогательные переменные */

/* _irq_stack:  .word DRAM_START + 0x00007F00 */
_irq_stack:  .word _start
_fiq_stack:  .word DRAM_START + 0x00004010
/*_svs_stack:  .word DRAM_START + 0x00004000 */
_svs_stack:  .word _start - 0x00004000
/* Адрес стартовой процедуры C-code */
_start_ccode: .word start_ccode 

/* Начало-конец BSS */
_bss_start:	.word __bss_start
_bss_end:	.word _end

/* Для копирования образа в точку линковки */
_copy_to:   .word _start
_copy_len:  .word __bss_start - _start  
_copy_run:  .word _code_start

/* Стек в статической памяти для вызова монитора защищенного режима */ 
SRAM_STACK:	.word 0x4020FFFC /*LOW_LEVEL_SRAM_STACK*/ 
 
/* CPSR - установка режимов работы процессора */

    .equ    NO_IRQ,         0x80                     /* mask to disable IRQ */ 
    .equ    NO_FIQ,         0x40                     /* mask to disable FIQ */ 
    .equ    NO_INT,         (NO_IRQ | NO_FIQ)        /*mask to disable IRQ and FIQ */ 
    .equ    FIQ_MODE,       0x11 
    .equ    IRQ_MODE,       0x12 
    .equ    SYS_MODE,       0x1F 
    .equ    SVS_MODE,       0x13    


/* Бит V регистра SCTRL CP15 (вектора в верхней памяти)*/
#define CR_V	(1 << 13)	/* Вектора в памяти 0xffff0000	*/  
#define OMAP3_GP_ROMCODE_API_WRITE_ACR		3

/*  Стартовая процедура, загрузка в адрес  0x80008000 */

reset:
    
	// Вызываем монитор для установки разрешения кэширования NEON в AuxControlReg
#ifdef SM_CALL	
	ldr	sp, SRAM_STACK
	
	mov r12, #OMAP3_GP_ROMCODE_API_WRITE_ACR /* Copy the Service ID in R12 */
	mov r0, #0x50 /* Copy parameter to R0 */
	mcr     p15, 0, r0, c7, c10, 4	@ DSB
	mcr     p15, 0, r0, c7, c10, 5	@ DMB
	/* Secure Monitor Call */
	.word	0xe1600070	/* SMC #0 */ 
#endif	
	
    	msr	cpsr,#(SVS_MODE | NO_INT)
     

					    
    /* Копирование в адрес загрузки	*/
	ldr r0, _copy_to
	add r1, pc, #(_start-checkpt-4) 
checkpt:	
	ldr r2, _copy_len
	add r2, r2, r0     // Конец приемника 
	
copy_loop:
	ldmia	r1!, {r9-r10}		/* copy from source address [r1]    */
	stmia	r0!, {r9-r10}		/* copy to   target address [r0]    */
	cmp	r0, r2			        /* until source end address [r2]    */
	blo	copy_loop	

	
    /* Переходим в зону копирования */
	ldr	pc, _copy_run
	
_code_start:	

	/* Устанавливаем стек для обработки IRQ и FIQ */
	msr	cpsr,#(IRQ_MODE | NO_INT)
	ldr r0, _irq_stack
	mov sp, r0
	msr	cpsr,#(FIQ_MODE | NO_INT)
	ldr r0, _fiq_stack
	mov sp, r0
	/* Устанавливаем режим SuperVisor и настраиваем стек для него */
	msr	cpsr,#(SVS_MODE | NO_INT)
	ldr r0, _svs_stack
	mov sp, r0

	/* Настройка таблицы векторов */
	/* Устанавливаем V=0 в CP15 SCTRL - VBAR указывает на вектор */
	
	mrc	p15, 0, r0, c1, c0, 0	@ Read CP15 SCTRL Register
	bic	r0, #CR_V		@ V = 0
	mcr	p15, 0, r0, c1, c0, 0	@ Write CP15 SCTRL Register
    
	/* Устанавливаем адрес таблицы векторов в CP15 VBAR */
	ldr	r0, =_start
	mcr	p15, 0, r0, c12, c0, 0	@Set VBAR

	/* Обнуляем BSS - зону неинициализированных данных */
	
	ldr	r0, _bss_start		
	ldr	r1, _bss_end		
	mov 	r2, #0
bss_loop:
	str	r2, [r0]
	cmp	r0, r1	
	add	r0, r0, #4
	bne	bss_loop   
	
	/* Вызываем C-code */ 
	ldr	pc, _start_ccode	

/* Обработчики исключений и прерываний */

	.align	5
undefined_instruction:
	msr	cpsr,#(SVS_MODE | NO_INT)
	bl	do_undefined_instruction

loklr: .word 0
	.align	5
software_interrupt:
 str lr, loklr
 bl	do_software_interrupt
	ldr lr, loklr
	movs   pc,r14
	
	/*
 movs   pc,r14
 stmfd sp!, {r0-r12, lr}
 
 ldr r0,[lr,#-4]
  
	bl	do_software_interrupt
	ldmfd sp!, {r0-r12, lr}
 movs   pc,r14
*/
	.align	5
prefetch_abort:
	msr	cpsr,#(SVS_MODE | NO_INT)
	bl	do_prefetch_abort

	.align	5
data_abort:
	msr	cpsr,#(SVS_MODE | NO_INT)
	bl	do_data_abort

	.align	5
not_used:
	msr	cpsr,#(SVS_MODE | NO_INT)
	bl	do_not_used

	.align	5
fiq:
	msr	cpsr,#(SVS_MODE | NO_INT)
	bl	do_fiq

	
	