/**
 *  @file symtbl.c
 *  @author Матяш А.В. (a.matiash@set-code.ru)
 *  @brief Заглушка для использования symTbl в коде.
 */

#include <symtbl.h>
#include <stdio.h>
#include <string.h>
#include <crt.h>
#include <help-design-elements.h>

// Поиск функций по шаблону - движок команды
static int findEngine (const char *template, bool caseSensitive) {
    helpHeader ("List of found commands");

    // Команды из консоли могут прийти с параметром 0 (т.е. без параметра).
    char dummy[] = "";
    if (template == 0) {
        template = dummy;
    }

    // Выбор функции поиска текста в строке
    static char * (*strCompareFunc)(const char *s1, const char *s2);
    if (caseSensitive)
        strCompareFunc = &strstr;
    else
        strCompareFunc = &stristr;

    // Поиск фрагмента в строке с учётом регистра или без.
    int n = 0;
    char idStr[20];
    for (unsigned int i=0; i<symTblSize; i++) {
        if ((*strCompareFunc) (symTbl[i].name, template) == NULL)
            continue;
        n++;
        sprintf (idStr, "%d", i);
        helpString (idStr, "-", symTbl[i].name);
        // printf ("% 6d  %s\n", i, symTbl[i].name);
    }

    // Завершающее описание.
    // printf ("--\n");
    helpBreakLine ();
    textColor (clNone);

    printf ("Found %d commands out of %d.\n", n, symTblSize);
    printf ("You can run the command using\n\"runf\" with a number from the list.\n");
    return OK;
}

// Регистрозависимый поиск функций по шаблону, выводи найденных имён.
int FIND (const char *template) {
    return findEngine (template, true);
}

// РегистроНЕзависимый поиск функций по шаблону, выводи найденных имён.
int find (const char *template) {
    return findEngine (template, false);
}

int runf (unsigned int number) {
    if (number >= symTblSize) return ERROR;
    int (*func) (void) = symTbl[number].ptr;
    (*func) ();
    return OK;
}

const char *findFuncName (unsigned int addr) {
    for (unsigned int i=0; i<symTblSize; i++) {
        if (symTbl[i].ptr == (void *) addr)
            return symTbl[i].name;
    }
    return "";
}


// Регистрозависимый поиск функций по шаблону и запуск найденных.
static void findAndPrint (const char *prefix, const char *template) {
    for (unsigned int i=0; i<symTblSize; i++) {
        if (strstr (symTbl[i].name, template) == NULL)
            continue;
        printf ("%s", prefix);
        int (*func)(void) = symTbl[i].ptr;
        (*func)();
    }
}

// Вывод в консоль всех коротких описаний библиотек.
int brief () {
    findAndPrint ("", "Brief");
    return 0;
}

// Вывод в консоль всех полных описаний библиотек.
int details () {
    findAndPrint ("--\n", "Details");
    return 0;
}
