/**
 * @addtogroup groupCore
 * @{
 * @file crc8.h
 * @brief Чек-сумма crc8.
 * @}
 */

#include "stdint.h"



/**
 * Расчитать чек-сумму crc8.
 * @param pData Указатель на данные, для которых ведется расчет чек-суммы.
 * @param size Размер данных.
 * @note Данные должны иметь размер не более 255 байт.
 * @return Чек-сумма crc8.
 */
uint8_t crc8_compute(const void* pData, uint8_t size);

/**
 * Получить промежуточный результат вычисления crc8.
 * @param byte Байт, который нужно добавить к чек-сумме.
 * @param crc8 Результат вычислений для предыдущего байта или 0xFF для инициализации.
 * @return Промежуточный результат вычисления crc8.
 */
uint8_t crc8_1step(uint8_t byte, uint8_t crc8);

