/**
    @addtogroup groupCore
    @{
    @file debug.h
    @brief Функции отладки
    @details Функции отладочного вывода в консоль.
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 04.04.2023

*/

#ifndef _DEBUG_H_
#define _DEBUG_H_

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  @name Измерение FPS
 *  @brief Запуск задачи измерения количества кадров в секунду.
 *  Результаты измерений выводятся в консоль раз в секунду.
 *  @{
 */

/**
 *  @brief Запуск задачи измерения \b FPS.
 *  @details Задачу измерения количества кадров в секунду
 *  следует запустить параллельно с запуском задачи 
 *  декодирования и вывода видео для контроля параметров
 *  декодирования.
 *  
 *  @param priority Приоритет задачи измерения.
 *  Чем меньше значение, тем выше приоритет задачи.
 *  Рекомендованное значение от 0 до 10.
 */
void fpsMeasurementStart (int priority);

/**
 *  @brief Увеличение счётчиков измерения \b FPS.
 *  @details Увеличение счётчиков следует 
 *  производить после декодирования каждого 
 *  кадра видео потока и смены активной 
 *  экранной области.
 */
void fpsMeasurementInc ();

/**
 *  @brief Текущее значение \b FPS.
 *  @details 
 *  
 *  @return unsigned int 
 */
unsigned int fpsValue ();

/**
 *  @brief Завершение задачи измерения \b FPS.
 *  @details Остановка задачи, прекращение вывда 
 *  в консоль.
 */
void fpsMeasurementStop ();
/// @}



/// @privatesection

/**
 *  @brief Чтение регистров процессора.
 *  @details Применялось при отладки DE2 (V3s).
 *  
 */
void readRegistrDumps ();

#ifdef __cplusplus
}
#endif

#endif
