/**
 *  @file uarttest.c
 *  @author Матяш А.В. (a.matiash@set-code.ru)
 *  @brief Эхо-тест COM-портов.
 *  @date 25.12.2023
 */

#include <arch.h>
#include <iolib.h>
#include <multex.h>
#include <string.h>
#include <tasklib.h>
#include <uart.h>

// Дескрипторы устройств UART (с запасом)
static int uartFd[100];
static bool uartFdInited = false;

int uartTest (int n) {
    char str[20];
    bool ok = false;
    sprintf (str, "UART-GPIO-%d", n);
    int uartGpio = archGetInt (str, &ok);
    if (!ok) {
        printf ("ERROR: To perform the test, you should define a UART-GPIO-%d set of lines in .arc the file.\n", n);
        return ERROR;
    }

    int maxLen = 16;
    char dataTx[maxLen];
    char dataRx[maxLen];
    for (int i = 0; i < maxLen; i++) {
        dataTx[i] = i;
        dataRx[i] = 0;
    }

    printf ("UART%d Test:\n", n);
    taskDelay (10);

    // Инит дискрипторов - 1 раз
    if (!uartFdInited) {
        memset (uartFd, -1, sizeof (uartFd));
        uartFdInited = true;
    }

    // Если порт ещё не открыт - открыть (далее оставить открытым).
    if (uartFd[n] < 0) {
        STATUS initResult = uartInit (n, uartGpio);
        if (initResult != OK) {
            printf ("  Init ERROR!\n");
            return ERROR;
        }

        uartSetBaudRate (n, UART_BAUD_115200);
        uartSetReadTimeout (n, 100);
        uartSetWriteTimeout (n, NO_WAIT);
        uartSetWriteBufferSize (n, maxLen * 2);
        uartSetReadBufferSize (n, maxLen * 2);

        uartFd[n] = open (uartName (n), O_RDWR);

        if (uartFd[n] < 0) {
            printf ("  Open device ERROR! 0x%08X\n", uartFd[n]);
            return ERROR;
        } else {
            printf ("  The %s device has been opened successfully\n", uartName (n));
        }
    }

    // Запись - чтение - проверка
    int len = write (uartFd[n], dataTx, maxLen);
    if (len != maxLen) {
        printf ("  Tx length ERROR! Tx=%d Rx=%d\n", maxLen, len);
        return ERROR;
    }
    len = read (uartFd[n], dataRx, maxLen);
    if (len != maxLen) {
        printf ("  Rx length ERROR! Tx=%d Rx=%d\n", maxLen, len);
        return ERROR;
    }

    for (int i = 0; i < maxLen; i++) {
        if (dataRx[i] != dataTx[i]) {
            printf ("  Data receive ERROR in byte %d\n", i);
            return ERROR;
        }
    }

    printf ("  %d bytes were successfully transmitted!\n", len);

    return OK;
}