#include <multex.h>
#include <tasklib.h>
#include <multimedia/softgraph.h>
#include <multimedia/de2.h>
#include <debug.h>
#include <multimedia/fonts.h> 
#include <stdio.h>
#include <msgqlib.h>
#include <string.h>
#include <gpio.h>
#include <stdlib.h>


#define DATE_TEXT_LENGTH    64
#define DATE_BACK_COLOR     0x803a3a3a
#define DATE_BORDER_COLOR   0x00ffffff
#define DATE_FONT_COLOR     0xffffffff
#define DATE_FONT_SIZE      8

#define CURSOR_BACK_COLOR   0x803a3a3a
#define CURSOR_BORDER_COLOR 0xff0000ff

#define FPS_BACK_COLOR      0x803a3a3a
#define FPS_BORDER_COLOR    0xffffffff
#define FPS_FONT_COLOR      0xffffffff
#define FPS_FONT_SIZE       18


static bool continueViewing = false;
static bool continueMoving = true;
static MSG_Q_ID msgDate = NULL;
static MSG_Q_ID msgTime = NULL;
static MSG_Q_ID msgCursor = NULL;

typedef struct {
    bool start;
    int x;
    int y;
} tCoordinates;

/// [softGraph example func]
void startScreen () {
    tScreenDeviceMode mode;
    sDisplayInfo display;


    // Заполнение описания экрана стандартными значениями
    de2GetDefaultScreenMode (&mode,
                             screenType_TM043NBH02);

    // Инициализация программной и аппаратной части
    display.width  = mode.xres;
    display.height = mode.yres;
    display.bpp    = mode.bpp;
    softGraphInit (&display,
                   de2Init (&mode, ovDataFormat_ARGB_8888)
                   );
    
    // Загрузка изображения
    HDCp bkgr;
    bkgr = loadFromJPG ("surf/wall_272.jpg");

    // Вывод изображения на экран
    sfDraw (softGraphConstr (), 0, 0, bkgr);

    // Ожидание окончания текущего буфера на экран
    de2WaitVerticalRetrace ();

    // Смена экранной и конструируемой поверхностей
    softGraphConstrUpdate (
        de2FlipScreenAndConstr ()
        );

    // Включение подсветки - яркость 80%
    de2SetBacklight (80);

    // Удаление временных объектов
    freeSurface (bkgr);
}
/// [softGraph example func]


typedef struct {
    HDCp surf;
    int x;
    int y;
    int dx;
    int dy;
} tSurfMove;

static void nextMovePosition (HDCp con, tSurfMove *move) {
    if (move->x > (int)con->sfWidth) if (move->dx > 0) move->dx = -move->dx;
    if (move->y > (int)con->sfHeight) if (move->dy > 0) move->dy = -move->dy;
    if (move->x < -((int)move->surf->sfWidth)) if (move->dx < 0) move->dx = -move->dx;
    if (move->y < -((int)move->surf->sfHeight)) if (move->dy < 0) move->dy = -move->dy;
    move->x += move->dx;
    move->y += move->dy;
}

static bool checkButton (const sTextRect *rect, const tCoordinates *cursor) {
    if (cursor->x < rect->x) return false;
    if (cursor->y < rect->y) return false;
    if (cursor->x > (rect->x + rect->w)) return false;
    if (cursor->y > (rect->y + rect->h)) return false;
    return true;
}

static bool invertMoveIfSelected (const tCoordinates *cursor, tSurfMove *move) {
    int interaction = rand ();
    int r = (int)move->surf->sfHeight / 4;
    tCoordinates center;
    center.x = move->x + ((int)move->surf->sfWidth / 2);
    center.y = move->y + ((int)move->surf->sfHeight / 2);
    if (cursor->x > (center.x + r)) return false;
    if (cursor->x < (center.x - r)) return false;
    if (cursor->y > (center.y + r)) return false;
    if (cursor->y < (center.y - r)) return false;

    switch (interaction % 7) {
        case 0:
            move->dx = -move->dx;
            break;
        case 1:
            move->dy = -move->dy;
            break;
        default:
        case 2:
            move->dx = -move->dx;
            move->dy = -move->dy;
            break;
        case 3:
            move->dx++;
            move->dy = -move->dy;
            break;
        case 4:
            move->dx--;
            move->dy = -move->dy;
            break;
        case 5:
            move->dy++;
            move->dx = -move->dx;
            break;
        case 6:
            move->dy--;
            move->dx = -move->dx;
            break;
    }
    return true;    
}


static void printText (
    HDCp con,
    const char *text, 
    pTtfFont font,
    unsigned int backColor,
    unsigned int borderColor,
    sTextRect *rect) {

    if (strlen (text) == 0) return;

    // Расчёт ограничивающего прямоугольника
    ttf_GetTextRect (&rect->w, &rect->h, font, text);
    rect->w +=10;

    // Подложка
    sfRectangle (con, rect->x, rect->y, rect->w, rect->h, backColor, borderColor);
    // Вывести надпись по центру
    ttf_PrintRect (con, rect, alignHCenter | alignVMiddle, font, text);

}

static void printButton (
    HDCp con,
    const char *text, 
    pTtfFont font,
    unsigned int backColor,
    unsigned int borderColor,
    sTextRect *rect) {

    if (strlen (text) == 0) return;

    // Подложка
    sfRectangle (con, rect->x, rect->y, rect->w, rect->h, backColor, borderColor);
    // Вывести надпись по центру
    ttf_PrintRect (con, rect, alignHCenter | alignVMiddle, font, text);

}


#define OBJECT_MAX_COUNT 20
static int mObjectCount = 0;

// Изменение количества объектов влёт
void setCount (int n) {
    int tmp = n;
    if (tmp <= 0)
        tmp = 1;
    if (tmp >= OBJECT_MAX_COUNT)
        tmp = OBJECT_MAX_COUNT - 1;
    mObjectCount = tmp;
}

static bool mTouchDetected = false;

bool cursorTouchObject () {
    if (mTouchDetected) {
        mTouchDetected = false;
        return true;
    }
    return false;
}


static int softTestTask (int n) {
    int objectCount = 0;
    setCount (n);
    char tmp[100];
    char date[DATE_TEXT_LENGTH];
    char time[DATE_TEXT_LENGTH];
    tCoordinates coord;
    int scoreCnt = 0;

    memset (date, 0, DATE_TEXT_LENGTH);
    memset (time, 0, DATE_TEXT_LENGTH);
    memset (&coord, 0, sizeof (tCoordinates));


    HDCp bkgr;
    tSurfMove imgs [OBJECT_MAX_COUNT];
    bkgr = loadFromJPG ("surf/wall_272.jpg");

    // Очереди сообщений
    msgDate = msgQCreate (10, DATE_TEXT_LENGTH, MSG_Q_FIFO);
    msgTime = msgQCreate (10, DATE_TEXT_LENGTH, MSG_Q_FIFO);
    msgCursor = msgQCreate (100, sizeof (tCoordinates), MSG_Q_FIFO);

    // Шрифты
    pTtfFont fpsFont  = ttf_LoadFont ("fnt/nsans-md.ttf", FPS_FONT_SIZE, FPS_FONT_COLOR, ttf_NoGlyphSaving);
    pTtfFont dateFont  = ttf_LoadFont ("fnt/nsans-md.ttf", DATE_FONT_SIZE, DATE_FONT_COLOR, ttf_NoGlyphSaving);

    for (int i=0; i<OBJECT_MAX_COUNT; i++){
        imgs [i].surf = loadFromPNG ("surf/falcon.png");
        imgs [i].x = -imgs [i].surf->sfWidth;
        imgs [i].y = -imgs [i].surf->sfHeight;
        imgs [i].dx = i + 5;
        imgs [i].dy = i + 1;
    }

    while (continueViewing) {

        // Обновление текущего количества видимых объектов
        objectCount = mObjectCount;

        HDCp con = softGraphConstr ();

        sfDraw (con, 0, 0, bkgr);

        for (int i=objectCount-1; i>=0; i--) {
            int alpha = 255; // - (i*10);
            // if (alpha < 50)
                // alpha = 50;
            sfDrawTransparent (con,
                       imgs [i].x, imgs [i].y,
                       imgs [i].surf,
                       alpha);

            if (continueMoving)
                nextMovePosition (con, &imgs[i]);
        }

        // Надписи
        sTextRect tmpRect;

        // Надпись "Score"
        tmpRect.x = 20;
        tmpRect.y = con->sfHeight - 50;
        sprintf (tmp, "Score: %04d", scoreCnt);
        printText (con, tmp, fpsFont,
                   FPS_BACK_COLOR, FPS_BORDER_COLOR,
                   &tmpRect);

        // Надпись "fps"
        tmpRect.x = con->sfWidth - 120;
        tmpRect.y = con->sfHeight - 50;
        sprintf (tmp, "FPS: %03d", fpsValue ());
        printText (con, tmp, fpsFont,
                   FPS_BACK_COLOR, FPS_BORDER_COLOR,
                   &tmpRect);

        // Кнопка x3
        sprintf (tmp, "X3");
        // sprintf (tmp, "Score: %04d", scoreCnt);
        sTextRect btnX3;
        btnX3.w = 60;
        btnX3.h = 40;
        btnX3.x = tmpRect.x + tmpRect.w - btnX3.w;
        btnX3.y = 80;
        
        printButton (con, tmp, fpsFont,
                   FPS_BACK_COLOR, FPS_BORDER_COLOR,
                   &btnX3);

        // Кнопка x3
        sprintf (tmp, "X10");
        // sprintf (tmp, "Score: %04d", scoreCnt);
        sTextRect btnX10;
        btnX10.w = 60;
        btnX10.h = 40;
        btnX10.x = btnX3.x + btnX3.w - btnX10.w;
        btnX10.y = btnX3.y + btnX3.h + 20;
        printButton (con, tmp, fpsFont,
                   FPS_BACK_COLOR, FPS_BORDER_COLOR,
                   &btnX10);

        // Надпись "date"
        msgQReceive (msgDate, date, DATE_TEXT_LENGTH, NO_WAIT);
        tmpRect.x = con->sfWidth - 80;
        tmpRect.y = 10;
        printText (con, date, dateFont,
                   DATE_BACK_COLOR, DATE_BORDER_COLOR,
                   &tmpRect);

        // Надпись "time"
        msgQReceive (msgTime, time, DATE_TEXT_LENGTH, NO_WAIT);
        tmpRect.x = con->sfWidth - 80;
        tmpRect.y = 30;
        printText (con, time, dateFont,
                   DATE_BACK_COLOR, DATE_BORDER_COLOR,
                   &tmpRect);

        // Получение данных о курсоре
        bool cursorPresent = false;
        while (msgQNumMsgs (msgCursor) > 0) {
            if (msgQReceive (msgCursor, &coord, sizeof (tCoordinates), NO_WAIT) == OK)
                cursorPresent = true;
            if (coord.start) {
                // Взаимодействие объектов с курсором.
                // printf ("TS!");
                if (checkButton (&btnX3, &coord)) {
                    // Кнопка Х3
                    setCount (3);
                } else 
                if (checkButton (&btnX10, &coord)) {
                    // Кнопка Х10
                    setCount (10);
                } else 
                for (int i=0; i < objectCount; i++) {
                    // Разворот объектов
                    if (invertMoveIfSelected (&coord, &imgs[i])) {
                        mTouchDetected = true;
                        scoreCnt ++;
                        break;
                    }
                }
                coord.start = false;
            }
        }
        
        // Отрисовка курсора
        if (cursorPresent) {
            sfSmoothCircle (con, coord.x, coord.y, 20, CURSOR_BACK_COLOR, 1, CURSOR_BORDER_COLOR);
            // sfRectangle (con, coord.x - 20, coord.y - 20, 40, 40, CURSOR_BACK_COLOR, CURSOR_BORDER_COLOR);
            sfLine (
                con,
                coord.x - 30, coord.y,
                coord.x + 30, coord.y,
                CURSOR_BORDER_COLOR);
            sfLine (
                con,
                coord.x, coord.y - 30,
                coord.x, coord.y + 30,
                CURSOR_BORDER_COLOR);
        }

        // Ожидание окончания отрисовки предыдущего кадра
        de2WaitVerticalRetrace ();
        
        // Смена экранных областей
        softGraphConstrUpdate (
            de2FlipScreenAndConstr ()
        );


        fpsMeasurementInc ();

    }

    // Освобождение ресурсов
    freeSurface (bkgr);
    for (int i=0; i<OBJECT_MAX_COUNT; i++)
        freeSurface (imgs[i].surf);

    msgQDelete (msgDate);
    msgQDelete (msgTime);
    msgQDelete (msgCursor);
    msgDate = NULL;
    msgTime = NULL;
    msgCursor = NULL;
    ttf_FreeFont (fpsFont);
    ttf_FreeFont (dateFont);

    return 0;
}



void softTest (int objectsCount) {

    if (continueViewing) {
        return;
    }
    continueViewing = true;
    continueMoving = true;

    // Основная задача - отрисовка экрана
    taskSpawn ("softTest",  12, 0, 0, (FUNCPTR)softTestTask, objectsCount);

    // Задача измерения fps
    fpsMeasurementStart (8);
}

void softTestToggle () {
    continueMoving = !continueMoving;
}

void softTestStop () {
    fpsMeasurementStop ();
    continueViewing = false;
}

void softTestUpdateDate (char *text) {
    if (msgDate == NULL) return;
    msgQSend (msgDate, text, DATE_TEXT_LENGTH, NO_WAIT, MSG_PRI_NORMAL);
}

void softTestUpdateTime (char *text) {
    if (msgTime == NULL) return;
    msgQSend (msgTime, text, DATE_TEXT_LENGTH, NO_WAIT, MSG_PRI_NORMAL);
}

void softTestUpdateCoordinates (int x, int y, bool start) {
    tCoordinates c;
    c.x = x;
    c.y = y;
    c.start = start;
    if (msgCursor == NULL) return;
    msgQSend (msgCursor, &c, sizeof (tCoordinates), NO_WAIT, MSG_PRI_NORMAL);
}
