/*
Header file: ehci-hcd.h
*/
#ifndef EHCI_HCD_H_
#define EHCI_HCD_H_

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */

#define EHCI_NUM_QH (32)
#define EHCI_NUM_TD (64)

#include <io.h>
#include <usb.h>

struct ehci_irq_element
       {
	    struct usb_device *dev;
		struct QH *qh;
		struct qTD *td;
		void *buf; 			  // Указатель на буфер данных
		int length;           // Длина буфера данных
		uint32_t token;
		struct ehci_irq_element *next;
	   };

struct ehci_async_element
       {
	    struct usb_device *dev;
		int pipe;
		struct QH *qh;        // Заголовок очереди
		struct qTD *ftd;      // Первый дескриптор
		struct qTD *td;	      // Последний дескриптор
		void *buf; 			  // Указатель на буфер данных
		int length;           // Длина буфера данных
		SEM_ID sem;			  // Семафор синхронизации 
		unsigned int tick;    // Время запуска
		unsigned int timeout; // Таймаут готовности
		int act_len;
		int status;
		
		#define AF_DONE (1<<0) // Элемент готов к удалению
        #define AF_UNLINKED (1<<2) // Элемент разлинкован
        
		int flags;            // Состояние
		struct ehci_async_element *next;
		struct ehci_async_element *prev;
	   };

void ehci_disconnect(struct usb_device *dev);
void run_ohci (struct usb_device *dev);

int usb_lowlevel_init (void *hcd);
int usb_lowlevel_stop (void);

//struct ehci_irq_element *ehci_irq_list = NULL;	

// Список активных элементов асинхронного обмена
//struct ehci_async_element *ehci_async_list = NULL; // Голова
//struct ehci_async_element *ehci_async_tail = NULL; // Хвост


#ifdef __cplusplus
}
#endif
#endif // ifndef EHCI-HCD_H_
