/**
 * @addtogroup groupCore
 * @{
 * @file inputstr.h
 * @brief Функции для обработки введенных строк и работы с управляющими клавишами.
 * @}
 */

#ifndef INPUTSTR_H_
#define INPUTSTR_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

#define MAX_BUFFER_SIZE 4096

/**
 * Управляющая клавиша.
 */
typedef enum
{
    CTL_NONE = 0,    //!< Клавиша не найдена.
    CTL_RIGHT,       //!< Стрелка вправо.
    CTL_LEFT,        //!< Стрелка влево.
    CTL_UP,          //!< Стрелка вверх.
    CTL_DWN,         //!< Стрелка вниз.
    CTL_INS,         //!< Insert.
    CTL_DEL,         //!< Delete.
    CTL_HOME,        //!< Home.
    CTL_END,         //!< End.
    CTL_PGUP,        //!< Page up.
    CTL_PGDWN,       //!< Page down.
    ALT_B,           //!< Alt-B.
    ALT_D,           //!< Alt-D.
    ALT_F,           //!< Alt-F.
} CTL_KEY;

/**
 * Считать следующую управляющую клавишу.
 * @details Функцию следует вызывать после считывания ESC-клавиши.
 * @return Значение перечисления CTL_KEY.
 */
CTL_KEY getCtlKey(void);


///! @privatesection
// Обработка строки для терминала.
void inputString(char *S);


/**
 *  @brief Обработка ввода строки для редактора \b Edit.
 *  @details Обработка ввода строки для редактора \b Edit.
 *  
 *  @param S Редактируемая строка.
 *  @param thisIsCommand Вводимая строка является командой, иначе это строка текста файла.
 *  @return Код действия, если была введена команда, иначе 0.
 */
int inputEditString(char *S, bool thisIsCommand);			  

#ifdef __cplusplus
}
#endif
#endif // ifndef INPUTSTR_H_
