/**
    @addtogroup groupCore
    @{
    @file led.h
    @brief Подключение линий светодиодов к системным функциям.
    @}
 */

#ifndef LED_H_
#define LED_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/**
 *  @anchor ancSysLedsDefines
 *  @name Макросы определения номеров системных светодиодов
 *  @{
 */
#define SYS_LED_TIMER 0     ///< @details Номер светодиода, индицирующего работу системного таймера.
#define SYS_LED_DISK  1     ///< @details Номер светодиода, индицирующего работу с картой памяти.
/// @}


/**
 *  @brief Задать \b GPIO для системного светодиода.
 *  @details Функция активирует работу указанного системного светодиода.
 *  После вызова функции светодиод подключается к системным функциям.
 *  Номера светодиодов для системных функций указаны 
 *  в соответствующей группе \ref ancSysLedsDefines "макросов".
 *  
 *  @param n Номер светодиода из группы \ref ancSysLedsDefines "макросов".
 *  @param gpio Линия, к которой аппаратной подключен светодиод. Для удобства
 *  указания линий \b GPIO следует использовать соответствующие \ref ancGpioNames "макросы",
 *  например <b>P_G + 1</b> для линии PG1.
 *  @return \ref true при удачном подключении, иначе \ref false.
 */
bool sysSetLedGpio (unsigned int n, unsigned int gpio);


/// @privatesection


/**
 *  @brief Инициализация модуля.
 *  @details Считывание информации об используемых линиях \b GPIO.
 *  Функция используется системой --- пользователю
 *  вызывать не нужно.
 *  
 *  @return \ref OK в случае успешного выполнения, иначе \ref ERROR.
 */
STATUS ledArchInit ();

void led_on(int led);
void led_off(int led);
void led_toggle(int led);


#ifdef __cplusplus
}
#endif
#endif // ifndef LED_H_
