#ifndef _ASX340_H_
#define _ASX340_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <multex.h>

/// @todo Оформить документацию

// Режим ручной экспозиции
/// @todo Слишком длинные имена - менять. Должны начинаться на asx340.
typedef enum {
    ae_rule_exec_rule_avgy_algo_average = 0x00,      // Average scene brightness
    ae_rule_exec_rule_avgy_algo_static = 0x01,       // Static weighted-average scene brightness
    ae_rule_exec_rule_avgy_algo_front_light = 0x02,  // Evaluative average scene brightness with front-light detection
    ae_rule_exec_rule_avgy_algo_back_light = 0x03    // Evaluative average scene brightness with back-light detection
} eAsx340Rule_exec_rule_avgy_algo;

// Режимы работы камеры
typedef enum {
    ASX340_VGA_50P,  // VGA, 50p
    ASX340_VGA_60P   // VGA, 60p
} eAsx340Mode;

// STATUS asx340Init (eAsx340Mode mode);  // Инициализация камеры (пока есть только два режима)

/**
 *  @brief Создать устройство работы с камерой ASX340.
 *  @details 
 *  
 *  @param mode
 *  @param i2cPort 
 *  @param csiChannel
 *  @return Указатель на созданное устройство. 
 */
void *asx340DevCreate (eAsx340Mode mode, unsigned int i2cPort, unsigned int csiChannel);

/**
 *  @brief Удаление устройства, освобождение памяти.
 *  @details 
 *  
 *  @param dev Устройство захвата созданное с помощью asx340DevCreate().
 *  @return  
 */
STATUS asx340DevDelete (void *dev);


/// @todo Все ли эти функции нужны пользователю? Они уже используются в ините.
STATUS asx340ResetHard (void *dev);      // Сброс
STATUS asx340ResetSoft (void *dev);      // Сброс (да, их в этой камере два)
unsigned int asx340ReadId (void *dev);         // Чтение ID камеры (проверка наличия камеры на линии)
STATUS asx340PadSlewAdjust (void *dev);  // Настроить фронты сигналов

STATUS asx340ModeVga50 (void *dev);  // Режим работы камеры VGA, 50p
STATUS asx340ModeVga60 (void *dev);  // Режим работы камеры VGA, 60p

STATUS asx340AeRuleCtrl (void *dev, eAsx340Rule_exec_rule_avgy_algo aeRule);  // Настройка режима авт.экспозиции
STATUS asx340AeManual (void *dev, unsigned int on_off);

// Размер кадра
unsigned int asx340FrameWidth (void *dev);
unsigned int asx340FrameHeight (void *dev);

// Используемое устройство CSI
void *asx340CsiDevice (void *dev);

/**
 *  @brief Ожидание получения кадра от камеры.
 *  @details Ожидание основано на функции csiWaitFrame()
 *  используемого канала \b CSI.
 *  
 *  @param dev Устройство захвата созданное с помощью asx340DevCreate().
 *  @return Результат работы csiWaitFrame().
 */
int asx340WaitFrame (void *dev);

/**
 *  @brief Сброс ожидания выбранного устройства.
 *  @details Функция сбрасывает флаг ожидания кадра выбранного устройства.
 *  При этом задача ожидающая очередного кадра на asx340WaitFrame() получит управление.
 *  
 *  @param dev Устройство захвата созданное с помощью asx340DevCreate().
 *  @return \ref OK при удачном сбросе флага, иначе \ref ERROR.
 */
STATUS asx340DevFlush (void *dev);

#ifdef __cplusplus
}
#endif

#endif

