/**
    @addtogroup groupMultimedia
    @{
    @file fontsapi.h
    @brief Интерфейс взаимодействия с библиотекой шрифтов.
    @details Для подключения библиотеки шрифтов данный интерфейс должен быть реализован
     в используемой библиотеке графики.
    @}
 */

/*
    @author Матяш А.В. (a.matiash@set-code.ru)
    @date 16.05.2023
*/


#ifndef _FONTS_API_H_
#define _FONTS_API_H_

#ifdef __cplusplus
extern "C" {
#endif

/**
 *  @brief Изображение символа - глиф.
 */
typedef struct {
    unsigned int *data;  ///< @details Указатель на буфер данных изображения.
    unsigned int w;      ///< @details Ширина в пикселях.
    unsigned int h;      ///< @details Высота в пикселях.
    unsigned int format; ///< @details Используется в аппаратных декодерах. @see \ref g2d_data_fmt.
} tGlyph;


/**
 *  @brief Копирование символа шрифта.
 *  @details Копирование производится на указанную поверхность
 *  
 *  @param dst Указатель на целевую поверхность. В зависимости от реализации библиотеки графики
 *  поверхность может быть описана разными структурами. В библиотеке целевая поверхность не используется.
 *  @param dstX,dstY Координаты копирования на целевой поверхности.
 *  @return \ref OK в случае удачного копирования, иначе \ref ERROR или код ошибки.
 */
int fntGlyphCopy (void *dst, int dstX, int dstY, tGlyph *glyph);

#ifdef __cplusplus
}
#endif

#endif
