/*
   RTOS MULTEX KLOTHO
   
   NET DEVICE
   
   Author: Sergey V.Sokolov
   05.2003
   
*/
#ifndef NETDEVICE_H_
#define NETDEVICE_H_

#include <multex.h>

#ifdef __cplusplus
extern "C" {
#endif
/* Public declarations */

// IP адрес устройства
typedef UINT ip_addr_t;
// MAC адрес устройства
typedef UCHAR mac_addr_t[6];
 
/*
 * Structure of a 10-100Mb/s Ethernet header.
 */
#define ETHERTYPE_PUP 0x0002   /* PUP protocol */
#define ETHERTYPE_IP  0x0008   /* IP protocol */
#define ETHERTYPE_ARP 0x0608  /* Addr. resolution protocol */
typedef struct ether_header
    {
     mac_addr_t ether_dhost;
     mac_addr_t ether_shost;
     USHORT     ether_type;
    } ETH_HDR;
// Размер указан явно из-за выравнивания структуры
#define SIZEOF_ETHERHEADER      0xe
// Минимальная длина пакета 
#define ETHERSMALL 60 

// Широковещательный пакет
#define FILL_BROADCAST(x) {int i; for(i=0;i<6;i++) x[i]=0xFF;}
#define FILL_ZEROCAST(x) {int i; for(i=0;i<6;i++) x[i]=0;}
// Свой MAC адрес
#define FILL_LOCALHOST(x) {int i;for(i=0;i<6;i++)x[i]=ndev->mac_addr[i];}
#define FILL_LOCALIP(x) x=ndev->ip_addr



#define ntohl(x) ((((x) & 0x000000ff) << 24) | \
    (((x) & 0x0000ff00) <<  8) | \
    (((x) & 0x00ff0000) >>  8) | \
    (((x) & 0xff000000) >> 24))

#define htonl(x) ((((x) & 0x000000ff) << 24) | \
    (((x) & 0x0000ff00) <<  8) | \
    (((x) & 0x00ff0000) >>  8) | \
    (((x) & 0xff000000) >> 24))

#define ntohs(x) ((((x) & 0x00ff) << 8) | \
    (((x) & 0xff00) >> 8))

#define htons(x) ((((x) & 0x00ff) << 8) | \
    (((x) & 0xff00) >> 8))


// Максимальное количество сетевых устройств в системе	
#define NET_DEV_MAX (4)
	

/*
   Драйвер сетевого устройства получает структуру netDevice при инициализации,
   создает блок управления и вписывает указатель на него в поле private;
   вписывает указатели на функции ResetFunc, StartFunc и OutputFunc.
   В обработчике прерывания по приему он вызывает функцию InputFunc, 
   которая назначается сверху
*/   
struct netDevice;
// Функция - обработчик прерывания по приему пакета
typedef STATUS (*netInputFunc)(struct netDevice *dev, ETH_HDR *hdr, void *data, size_t dataLen);
// Функция инициализации сетевой карты
typedef STATUS (*netDrvInit)(struct netDevice *dev);

// Структура сетевого устройства
typedef struct netDevice
       {
        unsigned int idx; // Индекс устройства в таблице
		
		void *private; // Данные для драйвера
        
		ip_addr_t ip_addr;
		ip_addr_t ip_mask; // Маска подсети
		ip_addr_t ip_gate; // Основной шлюз
        mac_addr_t mac_addr;         
        // Сброс адаптера                  
        STATUS (*ResetFunc)(struct netDevice *dev);
        // Старт адаптера                  
        STATUS (*StartFunc)(struct netDevice *dev);
        // Функция передачи пакета в сеть ( устанавливается драйвером )
        STATUS (*OutputFunc)(struct netDevice *dev, ETH_HDR *hdr, char *data,
                          UINT dataLen);
        // Функция получения состояния соединения с сетью
		STATUS (*LinkStateFunc)(struct netDevice *dev);		
		
       }NET_DEVICE;
       
// Устройство по индексу
NET_DEVICE *NetDev(unsigned int idx);
// Инициализация сетевого устройства - функция создает структуру устройства
// и инициализирует драйвер
extern STATUS netDeviceInit(netDrvInit DrvInit);
extern STATUS netDeviceReset(unsigned int idx);
extern STATUS netDeviceStart(unsigned int idx);
extern STATUS netDeviceOutput(unsigned int idx, ETH_HDR *hdr, char *data,UINT dataLen);
extern STATUS netDeviceHook(netInputFunc Hook);

// Установка IP-адреса

extern UINT strtoip(const char *IPAddr);
extern void iptostr(ip_addr_t IP, char *ipstr);

//То же, что и strtoip, но проводит дополнительные проверки на валидность адреса. 0.0.0.0 и 255.255.255.255 считаются валидными.
//При возврате Error содержимое IPAddr_IntDst не определено
extern STATUS strtoip_s(const char *IPAddr_StrSrc, ip_addr_t *IPAddr_IntDst);

extern UINT setIPAddress(unsigned int idx, char *IPAddr); // например 127.0.0.1
extern UINT getIPAddress(unsigned int idx);

extern int showIPAddress(unsigned int idx);

extern int netLinkState(unsigned int idx); // Состояние соединения 0 - Fail, 1- Success

// Установить маску подсети и шлюз
extern int setNetGateway(unsigned int idx, ip_addr_t mask, ip_addr_t gate);

// Найти нужное устройство по IP
extern NET_DEVICE *FindNDev(ip_addr_t ip, ip_addr_t *d_ip);

 
extern netInputFunc NetInputFunc;

#define ETH_ALEN        6               /* Octets in one ethernet addr   */
#define ETH_HLEN        14              /* Total octets in header.       */
#define ETH_ZLEN        60              /* Min. octets in frame sans FCS */
#define ETH_FRAME_LEN   1514            /* Max. octets in frame sans FCS */
#define VLAN_HLEN       4               /* The additional bytes */ 
#define ETH_FCS_LEN     4               /* Octets in the FCS             */

// Описания перенесены в udp.h
void setPrimaryIpAddress(ip_addr_t ip);
ip_addr_t getPrimaryIpAddress(void);


#ifdef __cplusplus
}
#endif
#endif // ifndef NETDEVICE_H_
