/**
 *  @addtogroup groupStdTypes
 *  @{
 * @file stdarg.h
 * @brief Макросы для поддержки функций с неопределенным числом аргументов неопределенного типа.
 * @details См. стандарт C11 7.16.
 * @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.16</a>.
 *  @}
 */
/*
 * @author Sergey Sokolov
 * @author Dubovich Timofey
 */

#ifndef STDARG_H
#define STDARG_H

/**
 * Тип, позволяющий работать с макросами \ref va_start, \ref va_end, \ref va_arg и \ref va_copy.
 */
typedef __builtin_va_list va_list; 

/**
 * Инициализировать объект типа \ref va_list.
 * @param vaList Объект типа \ref va_list.
 * @param parmN Последний аргумент функции, стоящий перед ', ...'.
 */
#define va_start(vaList, parmN)	__builtin_va_start(vaList, parmN)

/**
 * Освободить объект типа \ref va_list.
 * @param vaList Объект типа \ref va_list.
 */
#define va_end(vaList)	__builtin_va_end(vaList)

/**
 * Макрос, позволяющий последовательно получать неопределенные аргументы функции.
 * @param vaList Объект типа \ref va_list.
 * @param type Тип аргумента, который следует извлечь.
 */
#define va_arg(vaList, type) __builtin_va_arg(vaList, type)

/**
 * Инициализировать объект типа \ref va_list и скопировать туда другой подобный объект.
 * @param vaListDest Объект типа \ref va_list, который будет проинициализирован.
 * @param vaListSrc Объект типа \ref va_list, данные которого будут скопированы в \b vaListDest.
 */
#define va_copy(vaListDest, vaListSrc)	__builtin_va_copy(vaListDest, vaListSrc)


#endif //STDARG_H
