/**
   @file stdbool.h
   @brief Стандартные логические типы данных.
   @details В файле описаны логические типы данных, появившиеся в новом стандарте \b C11.
   @see <a href="https://www.iso.org/standard/57853.html">C11 standard 7.18</a>.
 */

/*
  date Created on: 28 янв. 2021 г.
  author Dubovich Timofey
*/

#ifndef __STDBOOL_H_
#define __STDBOOL_H_


//#define bool  _Bool
/**
 *  @brief Определение типа \b bool.
 *  @details Стандарт требует, чтобы макрос \b bool был определен как \b _Bool (в
 *  имплементации используемого компилятора - 1 байт), но в исходном коде @osName 
 *  в течении очень долгого времени использовалось определение через 
 *  \b 4х байтный \b int. Соответственно, смена макроса всё сломает.
 *  В связи с этим сделана поправка для данного типа.
 */
#define bool  int

/**
 *  @brief Определение типа \b true.
 *  @details Определение типа \b true по стандарту \b C11.
 */
#define true  1

/**
 *  @brief Определение типа \b false.
 *  @details Определение типа \b false по стандарту \b C11.
 */
#define false 0

/**
 *  @brief Проверка наличия определения логических типов.
 *  @details Данный макрос можно использовать для проверки 
 *  наличия определения типов \b true и \b false.
 */
#define __bool_true_false_are_defined 1

#endif /* __STDBOOL_H_ */
