/**
    @addtogroup groupUSB
    @{
    @file usb_driver.h
    @brief Регистрация USB драйвера в @osName
    @details Файл содержит функции регистрации драйверов \b USB в ОС @osName.

    <b>Подключение</b>: @verbatim #include <usb_driver.h> @endverbatim
    \ref sectionConfig "config.h": @verbatim #define INCLUDE_USB @endverbatim
    \ref sectionMakefile "Makefile": @verbatim LIBRARIES += -l_usbsunxi @endverbatim

    @see Общее описание работы с \b USB в главе \ref pageUSB.
    @}
*/

#ifndef USB_DRIVER_H_
#define USB_DRIVER_H_

#ifdef __cplusplus
extern "C" {
#endif
#include "usb.h"


/**
 *  @brief Регистрация драйвера в системе.
 *  @details Функция позволяет зарегистрировать драйвер устройства
 *  USB в системе. Для этого драйвер должен содержать две функции:
 *  \b probe() и \b disсonnect(). Функция \b probe() должна убедиться
 *  (исследуя дескрипторы устройства), что драйвер подходит для данного
 *  устройства и настроить устройство для работы (выбрать интерфейс,
 *  конфигурацию и протокол). Функция \b disconnect() должна обеспечить
 *  освобождение всех ресурсов, созданных функцией \b probe().
 *  
 *  @param name Имя устройства USB.
 *  @param probe Функция \b probe.
 *  @param disconnect Функция отсоединения устройства.
 */
void usb_register_driver(char *name,
      int (*probe)(struct usb_device *dev),
      void (*disconnect)(struct usb_device *dev));


/// @privatesection
struct usb_driver
	   {
	    char *name;
		int (*probe)(struct usb_device *dev);
		void (*disconnect)(struct usb_device *dev);
		struct usb_driver *next;
	   };

int usb_find_driver(struct usb_device *dev);      

#ifdef __cplusplus
}
#endif
#endif // ifndef USB_DRIVER_H_
